#!/bin/zsh

# Copyright 2014,2015 Paul Hänsch
#
# This file is part of shcgi.
# 
# shcgi is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# shcgi is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with shcgi.  If not, see <http://www.gnu.org/licenses/>. 

declare -A _GET
declare -A _POST
declare -A _REF
declare -A _COOKIE

[ -z "$HTTP_REFERER" ] && HTTP_REFERER="./"

cgi_get() {  # parse HTTP GET string
  debug "== CGI DATA: GET =="
  printf '%s\n' "$QUERY_STRING" |tr '&' '\n' |while read query; do
    key="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\1:')"
    val="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\2:')"
    _GET[$key]="$(printf "$(printf %s "$val" |sed 's:+: :g;s:\\:\\\\:g;s:%:\\x:g')")"
    debug "_GET[$key] => $val"
  done
}

cgi_post() {  # parse HTTP POST string
  debug "== CGI DATA: POST =="
  sed -u 1q |tr '&' '\n' |while read query; do
    key="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\1:')"
    val="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\2:')"
    value="$(printf "$(printf %s "$val" |sed 's:+: :g;s:\\:\\\\:g;s:%:\\x:g;')")"
    n=''
    if [ -n "${_POST[$key$n]}" ]; then
      n=0
      while [ -n "${_POST[$key$n]}" ]; do n=$(($n + 1)); done
    fi
    _POST[$key$n]="$value"
    debug "_POST[$key$n] => $value"
  done
}

cgi_refdata() { # Parse GET data from referer
  debug "== CGI DATA: REFERER =="
  printf '%s\n' "$HTTP_REFERER" |cut -d'?' -f2- |tr '&' '\n' |while read query; do
    key="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\1:')"
    val="$(printf %s "$query" |sed -r 's:^([a-zA-Z0-9_-]+)=(.*)$:\2:')"
    _REF[$key]="$(printf "$(printf %s "$val" |sed 's:+: :g;s:\\:\\\\:g;s:%:\\x:g')")"
    debug "_REF[$key] => $val"
  done
}

cgi_cookie() { # Parse GET data from referer
  debug "== CGI DATA: COOKIE =="
  printf '%s\n' "$HTTP_COOKIE" |tr ';' '\n' |while read query; do
    key="$(printf %s "$query" |sed -r 's:^ *([a-zA-Z0-9_-]+)=(.*)$:\1:')"
    val="$(printf %s "$query" |sed -r 's:^ *([a-zA-Z0-9_-]+)=(.*)$:\2:')"
    _COOKIE[$key]="$(printf "$(printf %s "$val" |sed 's:+: :g;s:\\:\\\\:g;s:%:\\x:g')")"
    debug "_COOKIE[$key] => $val"
  done
}

urlsafe(){
  printf %s "$*" \
  | sed 's;%;%25;g;
         s;\?;%3f;g;
         s;#;%23;g;
         s;<;%3c;g;
         s;>;%3e;g;
         s;&;%26;g;
         s;";%22;g;
         s;'\'';%27;g;'
}

htmlsafe(){
  printf %s "$*" \
  | sed 's;<;\&lt\;;g;
         s;>;\&gt\;;g;
         s;&;\&amp\;;g;
         s;";\&quot\;;g;
         s;'\'';\&apos\;;g;'
}

redirect(){
  printf '%s\n\n' "Location: $*"
  exit 0
}

set_cookie(){
  case "$1" in
    session|0)	expire='';;
    ''|default)	expire="$(LANG=C date -d "+ 1 week" +'%a, %d %b %Y %T %Z')";;
    *)		expire="$(LANG=C date -d "$1" +'%a, %d %b %Y %T %Z' 2>&-)";;
  esac
  cookie="$2"
  
  printf 'Set-Cookie: %s' "$cookie"
  [ -n "$expire" ] && printf '; Expires=%s' "$expire" 
  [ $# -ge 3 ] && shift 2 && printf '; %s' "$@"
  printf '\n'
}
